// CKartenstapel.cpp: Implementierung der Klasse CKartenstapel.
//
//////////////////////////////////////////////////////////////////////

#include "CKartenstapel.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CKartenstapel::CKartenstapel()
{
	erste_karte=letzte_karte=0;
	anzkarten=0;
	
	// Standartwerte fr die Anzeige
	anzeige.x=0;
	anzeige.y=0;
	anzeige.center=false;
	anzeige.verdeckt=false;
	anzeige.vertikal=false;
	anzeige.z=25;
	
	KartenPosAktuell=true;
}

CKartenstapel::~CKartenstapel()
{
	strKarte* tmp=erste_karte;
	while(erste_karte)
	{
		tmp=erste_karte;
		erste_karte=tmp->next;
		delete tmp;
	}
}

//////////////////////////////////////////////////////////////////////
// Anzeige der Karten beeinflussen
//////////////////////////////////////////////////////////////////////
void CKartenstapel::setAnzeige(int x, int y, bool center, bool verdeckt, bool vertikal, int z)
{
	// Werte bernehmen
	anzeige.x=x;
	anzeige.y=y;
	anzeige.center=center;
	anzeige.verdeckt=verdeckt;
	anzeige.vertikal=vertikal;
	anzeige.z=z;

	KartenPosAktuell=false;
}

//////////////////////////////////////////////////////////////////////
// Die Karten des Stapels anzeigen
//////////////////////////////////////////////////////////////////////
void CKartenstapel::zeichnen()
{
	if(anzkarten)
	{
		// Position der Karten Aktualisieren
		if(!KartenPosAktuell) setNewKartenPos();

		// Karten zeichnen
		strKarte* karte=erste_karte;
		while(karte)
		{
			karte->karte->zeichnen(anzeige.verdeckt);
			karte=karte->next;
		}
	}
}

//////////////////////////////////////////////////////////////////////
// Die Karten des Stapels auf Klick prfen
//////////////////////////////////////////////////////////////////////
CSpielkarte* CKartenstapel::click(int x, int y)
{
	strKarte* karte=erste_karte;
	CSpielkarte* geklickt=0;
	while(karte)
	{
		if(karte->karte->click(x,y)) geklickt= karte->karte;
		karte=karte->next;
	}
	
	return geklickt;
}

//////////////////////////////////////////////////////////////////////
// eine Karte in den Stapel aufnehemen
//////////////////////////////////////////////////////////////////////
void CKartenstapel::getKarte(CSpielkarte *karte)
{
	// ist auch wirklich eine karte angegeben
	if(!karte) return;

	// Kartenhlle erzeugen
	strKarte* tmp= new strKarte;
	tmp->karte=karte;

	// Karte einreihen
	if(!erste_karte)
	{
		tmp->next=0;
		tmp->prev=0;
		erste_karte=letzte_karte=tmp;
	}
	else
	{
		tmp->prev=letzte_karte;
		tmp->next=0;
		letzte_karte->next=tmp;
		letzte_karte=tmp;
	}

	// Anzahl der Karten aktualisieren
	anzkarten++;
		
	KartenPosAktuell=false;
}

//////////////////////////////////////////////////////////////////////
// Karte aus dem Stapel herausnehemn
//////////////////////////////////////////////////////////////////////
CSpielkarte* CKartenstapel::giveKarte(CSpielkarte* karte)
{
	strKarte* tmp_karte=erste_karte;
	// Karte im Stapel suchen
	while(tmp_karte)
	{
		if(tmp_karte->karte==karte) break;
		tmp_karte=tmp_karte->next;
	}

	if(tmp_karte)
	{
		// Karte aus dem Stapel nehmen
		if(tmp_karte->prev) tmp_karte->prev->next=tmp_karte->next;
		else erste_karte=tmp_karte->next;

		if(tmp_karte->next) tmp_karte->next->prev=tmp_karte->prev;
		else letzte_karte=tmp_karte->prev;

		// Anzahl Karten verringern
		anzkarten--;

		// speicher der Hlle freigeben
		delete tmp_karte;

		// neu positionieren der Karten veranlassen
		KartenPosAktuell=false;

		// Karte ausgeben
		return karte;
	}

	return 0;
}

CSpielkarte* CKartenstapel::giveFirstKarte()
{
	if(erste_karte)
	{
		// erste Karte vom Stapel nehmen
		strKarte* tmp=erste_karte;
		erste_karte=tmp->next;
		// war es die letzte? -> letzte_karte auch = 0
		if(!erste_karte) letzte_karte=0;

		// Anzahl der Karten verinngern
		anzkarten--;
		
		// Karte merken
		CSpielkarte* karte=tmp->karte;

		// hlle der Karte freigeben 
		delete tmp;

		// Karte ausgeben
		return karte;
	}

	return 0;
}

CSpielkarte* CKartenstapel::giveRandomKarte()
{
	// Sonderflle abfangen
	if(anzkarten==0) return 0;
	if(anzkarten==1) return giveFirstKarte();

	// zufallszahl erzeugen
	int zufallszahl=1+(rand()%anzkarten);

	// wenn es die erste Karte ist
	if(zufallszahl==1) return giveFirstKarte();

	// ansonsten die Karte suchen und ausgeben
	strKarte* kartenpos=erste_karte;
	for(int i=1;i<zufallszahl;i++)
		kartenpos=kartenpos->next;
	return giveKarte(kartenpos->karte);
}


//////////////////////////////////////////////////////////////////////
// Die Positionierung der Karten neu justieren
//////////////////////////////////////////////////////////////////////
void CKartenstapel::setNewKartenPos()
{
	int i=0;
	int x=anzeige.x;
	int y=anzeige.y;
	int ausmas=0;

	// ausma der Anzeige bestimmen -> wichtig bei der centeranzeige
	if(anzeige.vertikal) ausmas=(anzkarten-1)*anzeige.z+KARTENHOEHE;
	else ausmas=(anzkarten-1)*anzeige.z+KARTENBREITE;

	strKarte* karte=erste_karte;
	while(karte)
	{
		if(anzeige.center)
		{
			if(anzeige.vertikal) y=anzeige.y-ausmas/2+i*anzeige.z;
			else x=anzeige.x-ausmas/2+i*anzeige.z;
		}
		else
		{
			if(anzeige.vertikal) y=anzeige.y+i*anzeige.z;
			else x=anzeige.x+i*anzeige.z;
		}

		karte->karte->setpos(x,y);
		karte=karte->next;
		i++;
	}
}

//////////////////////////////////////////////////////////////////////
// Wert der Hchsten bzw. Niedrigsten Karte einer bestimmten Farbe
//////////////////////////////////////////////////////////////////////
int CKartenstapel::getWertHoechsteKarte(int farbe)
{
	strKarte* karte=erste_karte;
	int max=-1;

	// hchste Karte dieser Farbe suchen
	while(karte)
	{
		if(karte->karte->getFarbe()==farbe)
			if(karte->karte->getWert()>max)
				max=karte->karte->getWert();

		karte=karte->next;
	}

	return max;
}

int CKartenstapel::getWertNiedrigsteKarte(int farbe)
{
	strKarte* karte=erste_karte;
	int min=SPK_WERT_ASS+1;

	// niedrigste Karte dieser Farbe suchen
	while(karte)
	{
		if(karte->karte->getFarbe()==farbe)
			if(karte->karte->getWert()<min)
				min=karte->karte->getWert();

		karte=karte->next;
	}

	// wenn keine Karte der Farbe da
	if(min>SPK_WERT_ASS) min=-1;

	return min;
}

//////////////////////////////////////////////////////////////////////
// Die Karten des Stapels sortieren nach Farbe und Wert
//////////////////////////////////////////////////////////////////////
void CKartenstapel::sortKarten()
{
	// wenn die anzahl der karten kleiner als 2 ->nichts machen
	if(anzkarten<2) return;

	// wichtige Variablen
	CKartenstapel* stapel=new CKartenstapel;	// hier gehen die Karten sortiert rein
	int kartenwert=-1;			// Wert der hchsten Karte
	CSpielkarte* karte=0;		// die hchste Karte (wird als nchstes vom stapel genommen)
	strKarte* tmp=erste_karte;	// aktuelle position im Stapel
	bool karte_gefunden=false;

	// alle vier farben durchgehen
	for(int i=0;i<4;i++)
	{
		do 
		{
			// hchste Karte der Farbe suchen
			tmp=erste_karte;
			karte=0;
			kartenwert=-1;
			karte_gefunden=false;
			while(tmp)
			{	
				if(tmp->karte->getFarbe()==i)
					if(kartenwert < tmp->karte->getWert())
					{
						karte=tmp->karte;
						kartenwert=karte->getWert();
						karte_gefunden=true;
					}
					tmp=tmp->next;
			}

			// Karte vom Stapel nehmen und auf Zwischenstapel legen
			if(karte_gefunden) stapel->getKarte(this->giveKarte(karte));

		}while(karte_gefunden);
	}

	// Zwischenstapel ist nun sortiert (hchste Karte zuerst)
	// umstapel auf den eigentlichen Kartenstapel

	while(stapel->getAnzKarten())
		this->getKarte(stapel->giveFirstKarte());

	// zischenstapel wieder freigeben
	delete stapel;
}

//////////////////////////////////////////////////////////////////////
// Die Karten des Stapels mischen
//////////////////////////////////////////////////////////////////////
void CKartenstapel::mixKarten(int anzmix)
{
	// Sonderflle abfangen
	if(anzkarten<2) return;
	if(anzmix<1) return;

	// ein Zwischenstapel erzeugen
	CKartenstapel* stapel=new CKartenstapel;

	// Karten mischen
	while(anzmix--)
	{
		//Zufallskarte Karten in den Zwischenstapel
		while(this->getAnzKarten())
			stapel->getKarte(this->giveRandomKarte());

		// wieder zurckstapel
		while(stapel->getAnzKarten())
			this->getKarte(stapel->giveFirstKarte());
	}

	// zwischenstapel wieder freigeben
	delete stapel;

}